<?php

namespace Drp\SimpleCms\Http\Controllers\Admin;

use Drp\SimpleCms\Contracts\ContentFactory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Routing\Controller;

class PreviewController extends Controller
{
    /**
     * @var \Drp\SimpleCms\ContentFactory
     */
    private $contentFactory;

    public function __construct(ContentFactory $contentFactory)
    {
        $this->contentFactory = $contentFactory;
    }

    public function enable(?string $versionId = null, ?string $contentType = null): RedirectResponse
    {
        $url = route(config('simple_cms.home_route'));


        if (!is_null($versionId) && !is_null($contentType)) {
            $content = $this->contentFactory->byTypeAndVersionId($contentType, $versionId);
            $url = $content->frontEndUrl();

            if ($content->isPublished()) {
                session()->flash(
                    'expiredPreviewLink',
                    trans('simpleCms::preview.already_published')
                );
            }
        }

        session([
            'previewMode' => true,
            'previewModeRedirectUrl' => $this->getPreviousUrl(),
        ]);

        return redirect($url);
    }

    public function disable(): RedirectResponse
    {
        session()->forget('previewMode');

        $redirectTo = session()->pull('previewModeRedirectUrl');

        if (empty($redirectTo)) {
            return redirect()->route(config('simple_cms.admin_route'));
        }

        return redirect($redirectTo);
    }

    private function getPreviousUrl(): string
    {
        if ($this->cameFromSite()) {
            return redirect()
                ->getUrlGenerator()
                ->previous(route(config('simple_cms.admin_route')));
        }

        return route(config('simple_cms.admin_route'));
    }

    private function cameFromSite(): bool
    {
        $parsed = parse_url(url()->previous());
        return request()->root() === $parsed['scheme'] . '://' . $parsed['host'];
    }
}
